#!/usr/bin/env node
var browserify = require('../');
var fs = require('fs');
var path = require('path');
var JSONStream = require('JSONStream');

var argv = require('optimist')
    .boolean(['deps','pack','ig','dg', 'im'])
    .alias('insert-globals', 'ig')
    .alias('detect-globals', 'dg')
    .alias('ignore-missing', 'im')
    .alias('ig', 'fast')
    .default('ig', false)
    .default('im', false)
    .default('dg', true)
    .argv
;

if (argv.h || argv.help || process.argv.length <= 2) {
    return fs.createReadStream(__dirname + '/usage.txt')
        .pipe(process.stdout)
        .on('close', function () { process.exit(1) })
    ;
}
if (argv.v || argv.verbose) {
    return console.log(require('../package.json').version);
}

var entries = argv._.concat(argv.e).filter(Boolean).map(function(entry) {
    return path.resolve(process.cwd(), entry);
});
var b = browserify(entries);

b.on('error', function (err) {
    console.error(err);
    process.exit(1);
});

[].concat(argv.i).concat(argv.ignore).filter(Boolean)
    .forEach(function (i) { b.ignore(i) })
;

[].concat(argv.r).concat(argv.require).filter(Boolean)
    .forEach(function (r) { b.require(r, { expose: r }) });
;

// resolve any external files and add them to the bundle as externals
[].concat(argv.x).concat(argv.external).filter(Boolean)
    .forEach(function (x) { b.external(path.resolve(process.cwd(), x)) })
;

if (argv.pack) {
    process.stdin.pipe(b.pack()).pipe(process.stdout);
    process.stdin.resume();
    return;
}

if (argv.deps) {
    var stringify = JSONStream.stringify();
    b.deps().pipe(stringify).pipe(process.stdout);
    return;
}

var bundle = b.bundle({
    detectGlobals: argv['detect-globals'] !== false && argv.dg !== false,
    insertGlobals: argv['insert-globals'] || argv.ig,
    ignoreMissing: argv['ignore-missing'] || argv.im
});

var outfile = argv.o || argv.outfile;
if (outfile) {
    bundle.pipe(fs.createWriteStream(outfile));
}
else {
    bundle.pipe(process.stdout);
}
